<?php


namespace FiloBlu\PaymentMethodFilter\Observer\Payment;

class MethodIsActive implements \Magento\Framework\Event\ObserverInterface
{

    protected $helper;
    protected $customerSessionFactory;
    protected $backendSessionFactory;

    public function __construct(
        \FiloBlu\PaymentMethodFilter\Helper\PaymentFilter $helper,
        \Magento\Customer\Model\SessionFactory $customerSessionFactory,
        \Magento\Backend\Model\Auth\SessionFactory $backendSessionFactory) {
        $this->helper = $helper;
        $this->customerSessionFactory = $customerSessionFactory;
        $this->backendSessionFactory = $backendSessionFactory;
    }

    /**
     * Execute observer
     *
     * @param \Magento\Framework\Event\Observer $observer
     * @return void
     */
    public function execute(
        \Magento\Framework\Event\Observer $observer
    ) {
        $result = $observer->getData('result');
        $method_instance = $observer->getData('method_instance');
        $quote = $observer->getData('quote');

        if (!is_a($quote, "\Magento\Quote\Api\Data\CartInterface")) {
            return;
        }

        //if total is zero, don't mess with "no payment information required" payment method
        if ($quote && $quote->getBaseGrandTotal() == 0) {
            return;
        }

        if ($this->helper->isFilterEnabled()) {
            $paymentMethodWhitelist = $this->helper->paymentMethodWhitelist();
            if (!empty($paymentMethodWhitelist)) {

                $filterConditions = $this->helper->getFilterconditions();
                if (empty($filterConditions)) {
                    if (!$this->checkMethodAgainstWhitelist($method_instance->getCode(), $paymentMethodWhitelist)) {
                        $result->setData('is_available', FALSE);
                    }
                } else {
                    foreach ($filterConditions as $condition) {
                        switch ($condition) {
                            case 'giftcard':
                                if ($this->helper->quoteContainsGiftcard($quote) && $this->checkMethodAgainstWhitelist($method_instance->getCode(), $paymentMethodWhitelist) == FALSE) {
                                    $result->setData('is_available', FALSE);
                                }
                                break;
                            case 'products': //Products in Cart
                                if ($this->helper->quoteContainsTriggerProducts($quote) && $this->checkMethodAgainstWhitelist($method_instance->getCode(), $paymentMethodWhitelist) == FALSE) {
                                    $result->setData('is_available', FALSE);
                                }
                                break;
                            case 'product_custom_options': //Product Custom Options in Cart
                                if ($this->helper->quoteContainsTriggerCustomOptions($quote) && $this->checkMethodAgainstWhitelist($method_instance->getCode(), $paymentMethodWhitelist) == FALSE) {
                                    $result->setData('is_available', FALSE);
                                }
                                break;
                            case '*': //"any"
                                if ($this->checkMethodAgainstWhitelist($method_instance->getCode(), $paymentMethodWhitelist) == FALSE) {
                                    $result->setData('is_available', FALSE);
                                }
                            case '':
                            default:
                                break;
                        }
                    }
                }
            }
        }

        if ($this->helper->isLoginAsCustomerFilterEnabled()) {
            //let's hide selected payment methods for standard customers
            $customerSession = $this->customerSessionFactory->create();
            $backendSession = $this->backendSessionFactory->create();
            if (is_null($backendSession->getUser())) {
                //it's not an order made in backend
                $paymentMethodsBlacklist = $this->helper->loginAsCustomerPaymentMethodBlacklist();
                if ($this->helper->isAdminLoggedInAsCustomer($customerSession) == FALSE && in_array($method_instance->getCode(), $paymentMethodsBlacklist)) {
                    $result->setData('is_available', FALSE);
                }
            }
        }

        if ($this->helper->isFilterByShippingMethodEnabled()) {
            $shippingMethod = $quote->getShippingAddress()->getShippingMethod();
            $filterShippingMethods = $this->helper->getFilterByShippingMethodMap();
            foreach($filterShippingMethods as $map_id => $field) {
                $shipping_code_filter = $field['shipping_code_filter'];
                if ($shippingMethod == $shipping_code_filter) {
                    if ($this->checkMethodAgainstWhitelist($method_instance->getCode(), $field['payment_code_filter']) == FALSE) {
                        $result->setData('is_available', FALSE);
                    }
                }
            }
        }

    }

    protected function checkMethodAgainstWhitelist($methodCode, $paymentMethodWhitelist) {
        return in_array($methodCode, $paymentMethodWhitelist);
    }
}